'use strict';

// Load modules

const Stream = require('stream');
const Ammo = require('..');
const Code = require('code');
const Hoek = require('hoek');
const Lab = require('lab');
const Wreck = require('wreck');


// Declare internals

const internals = {};


// Test shortcuts

const lab = exports.lab = Lab.script();
const describe = lab.describe;
const it = lab.it;
const expect = Code.expect;


describe('header()', () => {

    it('parses header (start)', (done) => {

        expect(Ammo.header('bytes=0-4', 10)).to.deep.equal([{ from: 0, to: 4 }]);
        done();
    });

    it('parses header (middle)', (done) => {

        expect(Ammo.header('bytes=1-5', 10)).to.deep.equal([{ from: 1, to: 5 }]);
        done();
    });

    it('parses header (-to)', (done) => {

        expect(Ammo.header('bytes=-5', 10)).to.deep.equal([{ from: 5, to: 9 }]);
        done();
    });

    it('parses header (from-)', (done) => {

        expect(Ammo.header('bytes=5-', 45000)).to.deep.equal([{ from: 5, to: 44999 }]);
        done();
    });

    it('parses header (beyond end)', (done) => {

        expect(Ammo.header('bytes=10-20', 15)).to.deep.equal([{ from: 10, to: 14 }]);
        done();
    });

    it('parses header (wrong unit)', (done) => {

        expect(Ammo.header('horses=1-5', 10)).to.equal(null);
        done();
    });

    it('parses header (flipped)', (done) => {

        expect(Ammo.header('bytes=5-1', 10)).to.equal(null);
        done();
    });

    it('parses header (missing =)', (done) => {

        expect(Ammo.header('bytes 1-5', 10)).to.equal(null);
        done();
    });

    it('parses header (missing to and from)', (done) => {

        expect(Ammo.header('bytes=-', 10)).to.equal(null);
        done();
    });

    it('parses header (multiple ranges)', (done) => {

        expect(Ammo.header('bytes=1-5,7-10', 10)).to.deep.equal([{ from: 1, to: 5 }, { from: 7, to: 9 }]);
        done();
    });

    it('parses header (overlapping ranges)', (done) => {

        expect(Ammo.header('bytes=1-5,5-10', 10)).to.deep.equal([{ from: 1, to: 9 }]);
        done();
    });
});

describe('Stream', () => {

    it('returns a subset of a stream', (done) => {

        const random = new Buffer(5000);
        const source = Wreck.toReadableStream(random);
        const range = Ammo.header('bytes=1000-4000', 5000);
        const stream = new Ammo.Stream(range[0]);

        Wreck.read(source.pipe(stream), {}, (err, buffer) => {

            expect(buffer.toString()).to.equal(random.slice(1000, 4001).toString());
            done();
        });
    });

    it('processes multiple chunks', (done) => {

        const TestStream = function () {

            Stream.Readable.call(this);
            this._count = -1;
        };

        Hoek.inherits(TestStream, Stream.Readable);

        TestStream.prototype._read = function (size) {

            this._count++;

            if (this._count > 10) {
                return;
            }

            if (this._count === 10) {
                this.push(null);
                return;
            }

            this.push(this._count.toString());
        };

        const range = Ammo.header('bytes=2-4', 10);
        const stream = new Ammo.Stream(range[0]);

        const source = new TestStream();
        Wreck.read(source.pipe(stream), {}, (err, buffer) => {

            expect(buffer.toString()).to.equal('234');
            done();
        });
    });
});
